unit MMMainFrm;
{------------------------------------------------------------------------------}
(*

  This demo requires TRichView 17.3 or newer.


  Mail merging application supporting the following field values:
  - multiline text;
  - colored multiline text
  Fields are any strings inside {}.
  Supported fields:
  Text:
    {NAME}, {ADDRESS1}, {ADDRESS2}, {CITY}, {ZIP}, {COUNTRY}, {FULL_ADDRESS},
    {STATUS}
    {FULL_ADDRESS} is multiline.
    {NAME} and {STATUS} are colored.

  This demo generates report for all records as a single document (separated by
  page breaks).

  v1.1 - compatibility with TRichView 17.3. InsertMultilineText is moved to
    a separate unit. InsertMultilineText supports tabs in values.
  v1.0 - initial release
*)
{------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVScroll, RichView, StdCtrls, RVStyle, RVTable, CRVData, RVItem,
  RVEdit, RVTypes;

{$I RV_Defs.inc}

type

  { Information about a person, used for text fields }
  TPerson = class (TCollectionItem)
    public
      Name, Address1, Address2, Zip, City, Country, Status: String;
  end;


  TForm1 = class(TForm)
    rvOutput: TRichView;
    rvsOutput: TRVStyle;
    rvsTemplate: TRVStyle;
    Label1: TLabel;
    Label2: TLabel;
    rve: TRichViewEdit;
    Button1: TButton;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    Persons: TCollection;
    Index: Integer;
    procedure LoadPersons;
    procedure CreateTemplate;
    function GetFieldValueFromDatabase(const FieldName: TRVUnicodeString): TRVUnicodeString;
    function GetValueColor(const FieldName, Value: TRVUnicodeString): TColor;
    function GetColoredTextStyleNo(RVStyle: TRVStyle; StyleNo: Integer;
      AColor: TColor): Integer;
    procedure FillFields(RVData: TCustomRVData; StartItemNo: Integer);
    procedure InsertField(RVData: TCustomRVData;
      var ItemNo, Pos: Integer; const FieldName: TRVUnicodeString);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  RVInsertMultilineText; // <- included in this project


{$R *.dfm}

{------------------------------------------------------------------------------}
{ This procedure loads file CUSTOMERS.TXT into Persons.
  In real applications data can be stored in some database }
procedure TForm1.LoadPersons;
var F: TextFile;
    i, Count: Integer;
    Person: TPerson;
begin
  AssignFile(F, ExtractFilePath(Application.ExeName)+'Persons.txt');
  Reset(F);
  try
    Readln(F, Count);
    for i := 0 to Count-1 do
    begin
      Person := Persons.Add as TPerson;
      Readln(F, Person.Name);
      Readln(F, Person.Address1);
      Readln(F, Person.Address2);
      Readln(F, Person.City);
      Readln(F, Person.Zip);
      Readln(F, Person.Country);
      Readln(F, Person.Status);
    end;
  finally
    CloseFile(F);
  end;
  Label3.Caption := IntToStr(Persons.Count) + ' personal records in the database';
end;
{------------------------------------------------------------------------------}
{ Creating a sample template. You can edit it }
procedure TForm1.CreateTemplate;
begin
   rve.Clear;
   rve.AddNL('To {NAME}', 0, 0);
   rve.AddNL('{FULL_ADDRESS}', 0, 0);
   rve.AddNL('Letter Sample', 1, 1);
   rve.AddNL('Hello {NAME}', 0, 0);
   rve.AddNL('Your test result: {STATUS}. Congratulations!', 0, 0);
   rve.AddNL('', 0, 0);
   rve.AddNL('With best regards,', 0, 0);
   rve.AddNL('your template generator.', 0, 0);
   rve.AddNL('', 0, 0);
   rve.Format;
end;
{------------------------------------------------------------------------------}
{
  This function returns the current text field value by the field name.
}
function TForm1.GetFieldValueFromDatabase(
  const FieldName: TRVUnicodeString): TRVUnicodeString;
var Person: TPerson;
begin
  Person := Persons.Items[Index] as TPerson;
  if FieldName='NAME' then
    Result := Person.Name
  else if FieldName='ADDRESS1' then
    Result := Person.Address1
  else if FieldName='ADDRESS2' then
    Result := Person.Address2
  else if FieldName='CITY' then
    Result := Person.City
  else if FieldName='ZIP' then
    Result := Person.ZIP
  else if FieldName='COUNTRY' then
    Result := Person.Country
  else if FieldName='FULL_ADDRESS' then
    Result := Person.Address1+#13+Person.Address2+#13+Person.City+' '+Person.ZIP+#13+Person.Country
  else if FieldName='STATUS' then
    Result := Person.Status
  else
    Result := '{error: illegal field code}';
end;
{------------------------------------------------------------------------------}
{
  This function returns color for the field value.
  clNone means the default (unchanged) color.
}
function TForm1.GetValueColor(const FieldName, Value: TRVUnicodeString): TColor;
begin
  if FieldName = 'NAME' then
    Result := clNavy
  else if FieldName = 'STATUS' then
  begin
    if SameText(Value, 'PASSED') then
      Result := clGreen
    else if SameText(Value, 'FAILED') then
      Result := clRed
    else
      Result := clNone;
  end
  else
    Result := clNone;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  Persons := TCollection.Create(TPerson);
  LoadPersons;
  CreateTemplate;
end;
{------------------------------------------------------------------------------}
procedure TForm1.FormDestroy(Sender: TObject);
begin
  Persons.Free;
  Persons := nil;
end;
{------------------------------------------------------------------------------}
{ Generating output }
procedure TForm1.Button1Click(Sender: TObject);
var Stream: TStream;
    ItemCount: Integer;
begin
  Index := 0; // index of the current person
  ItemCount := 0;
  rvOutput.Clear;
  while Index<Persons.Count do
  begin
    // copying rve to the end of rvOutput
    Stream := TMemoryStream.Create;
    rve.SaveRVFToStream(Stream, False);
    Stream.Position := 0;
    rvOutput.InsertRVFFromStream(Stream, ItemCount);
    Stream.Free;
    // processing the last added copy of template
    if rvOutput.ItemCount>ItemCount then
    begin
      // starting from new page
      if ItemCount>0 then
        rvOutput.PageBreaksBeforeItems[ItemCount] := True;
      // replacing field codes
      FillFields(rvOutput.RVData, ItemCount);
    end;
    ItemCount := rvOutput.ItemCount;
    inc(Index);
  end;
  rvOutput.Format;
end;
{==============================================================================}
{ The code below this line can be copied your application unchanged }
{------------------------------------------------------------------------------}
(*
  This function iterates through all items in RVData starting from StartItemNo,
  searches for strings inside {}, calls InsertField()
  and replaces text between {} (including {}) with colored text
  You can move this function to your application unchanged.
*)
procedure TForm1.FillFields(RVData: TCustomRVData; StartItemNo: Integer);
var i,j,r,c: Integer;
    table: TRVTableItemInfo;
    FieldName: TRVUnicodeString;
    s: TRVUnicodeString;
    FieldStart: Integer;
begin
  i := StartItemNo;
  while i<RVData.ItemCount do
  begin
    if RVData.GetItem(i) is TRVTableItemInfo then
    begin // recursive processing of tables
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.RowCount-1 do
        for c := 0 to table.ColCount-1 do
          if table.Cells[r,c]<>nil then
            FillFields(table.Cells[r,c].GetRVData, 0);
    end
    else if RVData.GetItemStyle(i)>=0 then
    begin
      s := RVData.GetItemTextW(i);
      FieldStart := 0;
      j := 1;
      while j <= Length(s) do
      begin
        if s[j]='{' then // may be this is a beginning of field code
          FieldStart := j
        else if (s[j]='}') and (FieldStart>0) then
        begin // field code is found
          FieldName :=  Copy(s, FieldStart + 1, j - FieldStart - 1);
          // deleting field code
          Delete(s, FieldStart, j - FieldStart + 1);
          RVData.SetItemTextW(i, s);
          // inserting field value
          j := FieldStart;
          InsertField(RVData, i, j, FieldName);
          if RVData.GetItemStyle(i)<0 then
            s := ''
          else
            s := RVData.GetItemTextW(i);
          FieldStart := 0;
          continue;
        end;
        inc(j);
      end;
    end;
    inc(i);
  end;
end;
{------------------------------------------------------------------------------}
{
  If AColor = clNone, just returning StyleNo.
  Otherwise, returning item in RVStyle.TextStyles having all properties of
  RVStyle.TextStyles[StyleNo] but Color = AColor.
  Adding this text style, if necessary.
}
function TForm1.GetColoredTextStyleNo(RVStyle: TRVStyle; StyleNo: Integer;
  AColor: TColor): Integer;
begin
  if (AColor = clNone) or (AColor = RVStyle.TextStyles[StyleNo].Color) then
  begin
    Result := StyleNo;
    exit;
  end;
  Result := RVStyle.TextStyles.FindStyleWithColor(StyleNo, AColor,
    RVStyle.TextStyles[StyleNo].BackColor);
  if Result < 0 then
  begin
    with RVStyle.TextStyles.Add do
    begin
      Color := AColor;
      Standard := False;
    end;
    Result := RVStyle.TextStyles.Count - 1;
  end;
end;
{------------------------------------------------------------------------------}
{
  This function inserts a field value in the position (RVData, ItemNo, Pos).
  After inserting, (RVData, ItemNo, Pos) is a position after the inserted item.
  You can move this function to your application unchanged.
}
procedure TForm1.InsertField(RVData: TCustomRVData;
  var ItemNo, Pos: Integer; const FieldName: TRVUnicodeString);
var
  Color: TColor;
  Value: TRVUnicodeString;
  StyleNo: Integer;
begin
  Value := GetFieldValueFromDatabase(FieldName);
  Color := GetValueColor(FieldName, Value);
  StyleNo := GetColoredTextStyleNo(RVData.GetRVStyle,
    RVData.GetItemStyle(ItemNo), Color);
  InsertMultilineText(RVData, ItemNo, Pos, Value, StyleNo);
end;



end.
